﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Globalization;
using System.Reflection;
using System.Transactions;
using System.Windows.Forms;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SboGui;
using Produmex.Foundation.SboGui.Attributes;
using Produmex.Foundation.SboGui.Controls;
using Produmex.Foundation.SboGui.Controls.ButtonComboBox;
using Produmex.Foundation.SboGui.Controls.ComboBox;
using Produmex.Foundation.SboGui.Controls.Matrix;
using Produmex.Foundation.SboGui.Data;
using Produmex.Foundation.SboGui.Utilities;
using Produmex.Foundation.SboGui.Forms;
using System.Threading;

namespace Produmex.Sbo.Logex.AddOn.Forms
{
	/// <summary>
	/// This is an extension on the sales order form of sbo
	/// </summary>
	[SboFormAttribute(SboFormLoadType.LogicOnly, SboFormType.SALES_ORDER)]
	public class PmxSalesOrderForm : SboForm
	{

		private enum ViewOptions
		{
			OpenPickListProposals = 0,
			OpenPickLists = 1,
			StockLocking = 2
		}

		/// <summary>
		/// Used for logging
		/// </summary>
		private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

		private static class Defs
		{
			public const string BTN_OK_ID = "1";


			public const string ORDER_LINES_MATRIX_ID = "38";

			public const string ORDER_LINES_ITEMCODE_COLUMN_ID = "1";
			public const string ORDER_LINES_QTY_COLUMN_ID = "11";
			public const string ORDER_LINES_WHSCODE_COLUMN_ID = "24";
			public const string TXT_CARD_CODE = "4";
			public const string LINK_FREIGHT_CHARGES = "91";

			public const string COMBOBOX_COPY_FROM_ID = "10000330";
			public const string BTN_GEN_PICKLIST_PROPOSAL = "PMX_0001";
			public const string BTN_OPEN_PICKLIST_PROPOSALS = "PMX_B_02";
			public const string BTN_OPEN_PICKLISTS = "PMX_B_03";
			public const string BTN_VIEW = "PMX_B_04";
			public const string BTN_SERIAL_NUMBER = "PMX_B_05";


			public const string COMBOBOX_CURRENCY_ID = "70";
			public const string TXT_NUMATCARD_ID = "14";

			public const string LBL_PICK_LIST_TYPE = "PMX_L_01";
			public const string CMB_PICK_LIST_TYPE = "PMX_E_02";



			public const int VERTICAL_SPACING = 16;


			/// <summary>
			/// 
			/// </summary>
			public static class FreightChargesForm
			{
				public const string MATRIX_ID = "3";

				public const string FREIGHT_COLUMN_ID = "1";
				public const string AMOUNT_COLUMN_ID = "3";
				public const string TAX_CODE_COLUMN_ID = "17";
			}



		}

		private SboMatrix m_orderLinesMatrix;
		private SboMatrixEditTextColumn m_orderLines_Uom2;
		private SboMatrixEditTextColumn m_orderLines_WhsCode;
		private SboMatrixEditTextColumn m_orderLines_FreeStock;

		private SboButton m_btnCreatePickListProposal;
		private SboButtonCombo m_btnView;



		private SboStaticText m_lblPickListType;
		private SboComboBox m_cmbPickListType;

		private SboEditText m_cardCodeEditText;

		private SboButton m_btnOK;



		private SboButton m_btnLinkedSerialNumber;





		private CultureInfo m_fmtProvGui = null;



		/// <summary>
		/// Initializes a new instance of the <see cref="PmxSalesOrderForm"/> class.
		/// </summary>
		/// <param name="parent">The parent.</param>
		/// <param name="formUID">The form UID.</param>
		/// <param name="parentForm">The parent form.</param>
		public PmxSalesOrderForm(SboAddOn parent, string formUID, SboForm parentForm)
			: base(parent, formUID, parentForm)
		{
		}

		/// <summary>
		/// Initializes this instance.
		/// Use this function to add items or to extend an existing form.
		/// </summary>
		public override void Initialize()
		{
			base.Initialize();

			m_fmtProvGui = (CultureInfo)this.Parent.Language.Clone();
			m_fmtProvGui.NumberFormat.NumberDecimalSeparator = this.Parent.CompanyAdminInfo.DecimalSeparator;
			m_fmtProvGui.NumberFormat.NumberGroupSeparator = this.Parent.CompanyAdminInfo.ThousandsSeparator;


			m_cardCodeEditText = base.Controls.Get<SboEditText>(Defs.TXT_CARD_CODE);

			m_btnOK = base.Controls.Get<SboButton>(Defs.BTN_OK_ID);
			m_btnOK.Click += new EventHandler<SboButtonEventArgs>(m_btnOK_Click);


			m_orderLinesMatrix = base.Controls.Get<SboMatrix>(Defs.ORDER_LINES_MATRIX_ID);
			m_orderLinesMatrix.ChooseFromList += new EventHandler<Produmex.Foundation.SboGui.Data.SboChooseFromListEventArgs>(m_orderLinesMatrix_ChooseFromList);
			m_orderLinesMatrix.Validate += new EventHandler<SboMatrixEventArgs>(m_orderLinesMatrix_Validate);

			m_orderLines_Uom2 = m_orderLinesMatrix.Columns.Get<SboMatrixEditTextColumn>("U_PMX_UOM2");
			// m_orderLines_Uom2.Editable = false;  // Must be editable, to be filled in by PmxSalesDocLineDetailForm

			m_orderLines_WhsCode = m_orderLinesMatrix.Columns.Get<SboMatrixEditTextColumn>(Defs.ORDER_LINES_WHSCODE_COLUMN_ID);
			m_orderLines_FreeStock = m_orderLinesMatrix.Columns.Get<SboMatrixEditTextColumn>("U_PMX_FREE");
			m_orderLines_FreeStock.Editable = false;


			SboComboBox btnCopyFrom = base.Controls.Get<SboComboBox>(Defs.COMBOBOX_COPY_FROM_ID);

			m_btnCreatePickListProposal = base.Controls.Add<SboButton>(Defs.BTN_GEN_PICKLIST_PROPOSAL);
			m_btnCreatePickListProposal.Size = new System.Drawing.Size(120, btnCopyFrom.Height);
			m_btnCreatePickListProposal.Location = btnCopyFrom.Location + new System.Drawing.Size(-125, 0);
			m_btnCreatePickListProposal.LinkTo = Defs.COMBOBOX_COPY_FROM_ID;
			m_btnCreatePickListProposal.Caption = "Create PickList Proposal";
			m_btnCreatePickListProposal.LocalizationKey = "Logex.AddOn.SalesOrderForm.BtnCreatePickListProposal";
			m_btnCreatePickListProposal.InternalSboItemObject.SetAutoManagedAttribute(SAPbouiCOM.BoAutoManagedAttr.ama_Editable, (int)SAPbouiCOM.BoAutoFormMode.afm_All, SAPbouiCOM.BoModeVisualBehavior.mvb_False);
			m_btnCreatePickListProposal.InternalSboItemObject.SetAutoManagedAttribute(SAPbouiCOM.BoAutoManagedAttr.ama_Editable, (int)SAPbouiCOM.BoAutoFormMode.afm_Ok, SAPbouiCOM.BoModeVisualBehavior.mvb_True);
			m_btnCreatePickListProposal.Click += new EventHandler<SboButtonEventArgs>(m_btnCreatePickListProposal_Click);


			// View button combo
			this.m_btnView = base.Controls.Add<SboButtonCombo>(Defs.BTN_VIEW);
			this.m_btnView.Size = new System.Drawing.Size(120, m_btnCreatePickListProposal.Height);
			this.m_btnView.Location = m_btnCreatePickListProposal.Location + new System.Drawing.Size(-125, 0);
			this.m_btnView.LinkTo = Defs.COMBOBOX_COPY_FROM_ID;
			this.m_btnView.Caption = "View";
			this.m_btnView.LocalizationKey = "Logex.AddOn.SalesForm.BtnView";
			this.m_btnView.AffectsFormMode = false;
			this.m_btnView.Select += new EventHandler<SboButtonComboBoxEventArgs>(m_btnView_Select);

			this.m_btnView.DisplayValidValueDescription = true;
			this.m_btnView.ValidValues.AddLocalizedDescription(((int)ViewOptions.OpenPickListProposals).ToString(), "Logex.AddOn.SalesForm.BtnOpenPickListProposals", this.Parent.ScreenLocalizer);
			this.m_btnView.ValidValues.AddLocalizedDescription(((int)ViewOptions.OpenPickLists).ToString(), "Logex.AddOn.SalesForm.BtnOpenPickLists", this.Parent.ScreenLocalizer);
			this.m_btnView.ValidValues.AddLocalizedDescription(((int)ViewOptions.StockLocking).ToString(), "Logex.AddOn.SalesForm.BtnPmxStockLocking", this.Parent.ScreenLocalizer);
			this.m_btnView.InternalSboItemObject.SetAutoManagedAttribute(SAPbouiCOM.BoAutoManagedAttr.ama_Editable, (int)SAPbouiCOM.BoAutoFormMode.afm_All, SAPbouiCOM.BoModeVisualBehavior.mvb_False);
			this.m_btnView.InternalSboItemObject.SetAutoManagedAttribute(SAPbouiCOM.BoAutoManagedAttr.ama_Editable, (int)SAPbouiCOM.BoAutoFormMode.afm_Ok, SAPbouiCOM.BoModeVisualBehavior.mvb_True);






			SboComboBox cmbCurrency = base.Controls.Get<SboComboBox>(Defs.COMBOBOX_CURRENCY_ID);
			SboEditText txtNumAtCard = base.Controls.Get<SboEditText>(Defs.TXT_NUMATCARD_ID);

			this.m_cmbPickListType = base.Controls.Add<SboComboBox>(Defs.CMB_PICK_LIST_TYPE);
			this.m_cmbPickListType.Location = new Point(txtNumAtCard.Location.X, cmbCurrency.Location.Y + Defs.VERTICAL_SPACING);
			this.m_cmbPickListType.Size = new Size(txtNumAtCard.Size.Width, txtNumAtCard.Size.Height);
			this.m_cmbPickListType.AffectsFormMode = true;
			this.m_cmbPickListType.DisplayValidValueDescription = true;

			//fill combobox
			m_cmbPickListType.ValidValues.Add("", "");
			m_cmbPickListType.ValidValues.Add("1", "1");
			m_cmbPickListType.ValidValues.Add("2", "2");
			m_cmbPickListType.ValidValues.Add("3", "3");

			this.m_cmbPickListType.BindToDataSource("ORDR", "U_PMX_PLTY");


			this.m_lblPickListType = base.Controls.Add<SboStaticText>(Defs.LBL_PICK_LIST_TYPE);
			this.m_lblPickListType.Location = new Point(cmbCurrency.Location.X, m_cmbPickListType.Location.Y);
			this.m_lblPickListType.Size = new Size(cmbCurrency.Size.Width, cmbCurrency.Size.Height);
			this.m_lblPickListType.LinkTo = Defs.CMB_PICK_LIST_TYPE;
			this.m_lblPickListType.Caption = "Pick list type";
			this.m_lblPickListType.LocalizationKey = "Logex.AddOn.SalesOrderForm.LblPickListType";




			m_btnLinkedSerialNumber = this.Controls.Add<SboButton>(Defs.BTN_SERIAL_NUMBER);
			m_btnLinkedSerialNumber.Width = 130;
			m_btnLinkedSerialNumber.Left = m_orderLinesMatrix.Left;
			m_btnLinkedSerialNumber.Top = m_orderLinesMatrix.Top + m_orderLinesMatrix.Height;
			m_btnLinkedSerialNumber.Caption = "Linked Serial numbers";
			m_btnLinkedSerialNumber.PaneLevel = m_orderLinesMatrix.PaneLevel;
			m_btnLinkedSerialNumber.LocalizationKey = "Logex.AddOn.SalesForm.BtnLinkedSerialNumbers";
			m_btnLinkedSerialNumber.InternalSboItemObject.SetAutoManagedAttribute(SAPbouiCOM.BoAutoManagedAttr.ama_Editable, (int)SAPbouiCOM.BoAutoFormMode.afm_All, SAPbouiCOM.BoModeVisualBehavior.mvb_False);
			m_btnLinkedSerialNumber.InternalSboItemObject.SetAutoManagedAttribute(SAPbouiCOM.BoAutoManagedAttr.ama_Editable, (int)SAPbouiCOM.BoAutoFormMode.afm_Ok, SAPbouiCOM.BoModeVisualBehavior.mvb_True);
			m_btnLinkedSerialNumber.Click += new EventHandler<SboButtonEventArgs>(m_btnLinkedSerialNumber_Click);





			// System.Text.StringBuilder strDump = new System.Text.StringBuilder();
			// Produmex.Foundation.Reflection.ReflectionHelper.DumpObject( null, null, this.ChooseFromLists, null, 10, strDump );
			// m_orderLinesMatrix.Columns.Get<SboMatrixEditTextColumn>( "U_OrgArt" ).ChooseFromListUID = "6";
			// m_orderLinesMatrix.Columns.Get<SboMatrixEditTextColumn>( "U_OrgArt" ).ChooseFromListAlias = "U_OrgArt";

#if DEBUG
			string formXml = this.InternalSboFormObject.GetAsXML();
#endif
		}



		void m_btnLinkedSerialNumber_Click(object sender, SboButtonEventArgs e)
		{

		}




		/// <summary>
		/// Called when [form loaded].
		/// </summary>
		/// <param name="sender">The sender.</param>
		protected override void OnFormLoaded(SboForm sender)
		{
			base.OnFormLoaded(sender);


			//Do NOT set this visible, because it will cause flickering conflict with other add-ons
			//this.Visible = true;
			try
			{
				//this.ActiveControl = m_cardCodeEditText;
				//this.InternalSboFormObject.ActiveItem = "4";
				this.InternalSboFormObject.Items.Item("4").Click(SAPbouiCOM.BoCellClickType.ct_Regular);
			}
			catch (Exception ex)
			{
				s_log.Error("Error while setting active control", ex);
			}
		}

		/// <summary>
		/// Called when [form activated].
		/// </summary>
		/// <param name="sender">The sender.</param>
		protected override void OnFormActivated(SboForm sender)
		{
			base.OnFormActivated(sender);
			//this.InternalSboFormObject.ActiveItem = "16";
			//this.InternalSboFormObject.Items.Item("16").Click(SAPbouiCOM.BoCellClickType.ct_Regular);
			m_cardCodeEditText.Value = "C_FO_002";
		}


		void m_btnView_Select(object sender, SboButtonComboBoxEventArgs e)
		{
		}





		/// <summary>
		/// Handles the Click event of the m_btnCreatePickListProposal control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="Produmex.Foundation.SboGui.Controls.SboButtonEventArgs"/> instance containing the event data.</param>
		void m_btnCreatePickListProposal_Click(object sender, SboButtonEventArgs e)
		{

		}

		/// <summary>
		/// Called when business object is loaded.
		/// </summary>
		/// <param name="sboForm">The sbo form.</param>
		/// <param name="e">The <see cref="Produmex.Foundation.SboGui.FormDataEventArgs"/> instance containing the event data.</param>
		protected override void OnDataLoad(SboForm sboForm, FormDataEventArgs e)
		{
			base.OnDataLoad(sboForm, e);


			RefreshOrderMatrixPmxFreeColumn(null);

		}

		/// <summary>
		/// Called when the form is resized
		/// </summary>
		protected override void OnResize()
		{
			// Freeze the form to avoid flickering
			using (SboForm.Freezer freezer = new SboForm.Freezer(this))
			{
				base.OnResize();

				//Check if the controls are initialized
				if (this.m_btnLinkedSerialNumber == null)
					return;

				m_btnLinkedSerialNumber.Top = m_orderLinesMatrix.Top + m_orderLinesMatrix.Height;
			}
		}

		void m_orderLinesMatrix_ChooseFromList(object sender, Produmex.Foundation.SboGui.Data.SboChooseFromListEventArgs e)
		{
		}

		void m_orderLinesMatrix_Validate(object sender, SboMatrixEventArgs e)
		{
		}

		/// <summary>
		/// Called when add toolbar button is pressed.
		/// </summary>
		/// <param name="beforeAction">if set to <c>true</c> this function is called before SAP Business One processes the event; otherwise, after.</param>
		protected override void OnAddPressed(bool beforeAction)
		{
			base.OnAddPressed(beforeAction);

		}





		/// <summary>
		/// Handles the Click event of the m_btnOK control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="Produmex.Foundation.SboGui.Controls.SboButtonEventArgs"/> instance containing the event data.</param>
		void m_btnOK_Click(object sender, SboButtonEventArgs e)
		{
		}



		private void RefreshOrderMatrixPmxFreeColumn(int? row)
		{

		}

	}
}
