using System;
using System.Reflection;
using System.Windows.Forms;
using Produmex.Foundation;
using Produmex.Foundation.Diagnostics;

namespace Produmex.Sbo.Logex.AddOn
{
    static class Program
    {
        /// <summary>
        /// Used for logging
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        [STAThread]
        static void Main( string[] args )
        {
            ExecuteAddOn( args, null );
        }

        /// <summary>
        /// Executes the add on.
        /// ! This function is called by the AddOnLoader, do not remove it or change the parameters !
        /// </summary>
        /// <param name="args">The args.</param>
        /// <param name="application">The application.</param>
        public static void ExecuteAddOn( string[] args, SAPbouiCOM.Application application )
        {
            ExecuteAddOn2( args, application, null );
        }

        /// <summary>
        /// Executes the add on.
        /// ! This function is called by the AddOnLoader, do not remove it or change the parameters !
        /// </summary>
        /// <param name="args">The args.</param>
        /// <param name="application">The application.</param>
        /// <param name="company">The company.</param>
        public static void ExecuteAddOn2( string[] args, SAPbouiCOM.Application application, SAPbobsCOM.Company company )
        {
            AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler( CurrentDomain_UnhandledException );
            Application.ThreadException += new System.Threading.ThreadExceptionEventHandler( Application_ThreadException );

            try
            {
                using ( PmxAddOn addOn = new PmxAddOn( "Produmex Logex", application, company ) )
                {
                    // Start Message Loop
                    System.Windows.Forms.Application.Run();
                }
            }
            catch ( ProdumexException ex )
            {
                if ( ex.Error.ErrorCode < Produmex.Foundation.Licensing.ProdumexErrorCodes.First
                    || ex.Error.ErrorCode > Produmex.Foundation.Licensing.ProdumexErrorCodes.Last )
                {
                    s_log.Error( "Error in Logex AddOn.", ex );
                    MessageBox.Show( ex.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error );
                }

                if ( application != null ) throw;
            }
            catch ( Exception ex )
            {
                s_log.Error( "Error in Logex AddOn.", ex );
                MessageBox.Show( ex.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error );

                if ( application != null ) throw;
            }
        }

        static void CurrentDomain_UnhandledException( object sender, UnhandledExceptionEventArgs e )
        {
            s_log.Error( "Error in Logex AddOn.", e.ExceptionObject as Exception );

            if ( e.ExceptionObject is Exception )
            {
                MessageBox.Show( ( (Exception)e.ExceptionObject ).ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error );
            }
        }

        static void Application_ThreadException( object sender, System.Threading.ThreadExceptionEventArgs e )
        {
            s_log.Error( "Error in Logex AddOn.", e.Exception );

            MessageBox.Show( e.Exception.ToString(), "Error", MessageBoxButtons.OK, MessageBoxIcon.Error );
        }


    }
}
